# Copyright 2020, Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

package Google::Ads::GoogleAds::V18::Enums::ExtensionTypeEnum;

use strict;
use warnings;

use Const::Exporter enums => [
  UNSPECIFIED        => "UNSPECIFIED",
  UNKNOWN            => "UNKNOWN",
  NONE               => "NONE",
  APP                => "APP",
  CALL               => "CALL",
  CALLOUT            => "CALLOUT",
  MESSAGE            => "MESSAGE",
  PRICE              => "PRICE",
  PROMOTION          => "PROMOTION",
  SITELINK           => "SITELINK",
  STRUCTURED_SNIPPET => "STRUCTURED_SNIPPET",
  LOCATION           => "LOCATION",
  AFFILIATE_LOCATION => "AFFILIATE_LOCATION",
  HOTEL_CALLOUT      => "HOTEL_CALLOUT",
  IMAGE              => "IMAGE"
];

1;
