# Copyright 2020, Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

package Google::Ads::GoogleAds::V18::Enums::AssetFieldTypeEnum;

use strict;
use warnings;

use Const::Exporter enums => [
  UNSPECIFIED              => "UNSPECIFIED",
  UNKNOWN                  => "UNKNOWN",
  HEADLINE                 => "HEADLINE",
  DESCRIPTION              => "DESCRIPTION",
  MANDATORY_AD_TEXT        => "MANDATORY_AD_TEXT",
  MARKETING_IMAGE          => "MARKETING_IMAGE",
  MEDIA_BUNDLE             => "MEDIA_BUNDLE",
  YOUTUBE_VIDEO            => "YOUTUBE_VIDEO",
  BOOK_ON_GOOGLE           => "BOOK_ON_GOOGLE",
  LEAD_FORM                => "LEAD_FORM",
  PROMOTION                => "PROMOTION",
  CALLOUT                  => "CALLOUT",
  STRUCTURED_SNIPPET       => "STRUCTURED_SNIPPET",
  SITELINK                 => "SITELINK",
  MOBILE_APP               => "MOBILE_APP",
  HOTEL_CALLOUT            => "HOTEL_CALLOUT",
  CALL                     => "CALL",
  PRICE                    => "PRICE",
  LONG_HEADLINE            => "LONG_HEADLINE",
  BUSINESS_NAME            => "BUSINESS_NAME",
  SQUARE_MARKETING_IMAGE   => "SQUARE_MARKETING_IMAGE",
  PORTRAIT_MARKETING_IMAGE => "PORTRAIT_MARKETING_IMAGE",
  LOGO                     => "LOGO",
  LANDSCAPE_LOGO           => "LANDSCAPE_LOGO",
  VIDEO                    => "VIDEO",
  CALL_TO_ACTION_SELECTION => "CALL_TO_ACTION_SELECTION",
  AD_IMAGE                 => "AD_IMAGE",
  BUSINESS_LOGO            => "BUSINESS_LOGO",
  HOTEL_PROPERTY           => "HOTEL_PROPERTY",
  DEMAND_GEN_CAROUSEL_CARD => "DEMAND_GEN_CAROUSEL_CARD"
];

1;
