# Copyright 2020, Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

package Google::Ads::GoogleAds::V18::Enums::UserListCustomerTypeCategoryEnum;

use strict;
use warnings;

use Const::Exporter enums => [
  UNSPECIFIED            => "UNSPECIFIED",
  UNKNOWN                => "UNKNOWN",
  ALL_CUSTOMERS          => "ALL_CUSTOMERS",
  PURCHASERS             => "PURCHASERS",
  HIGH_VALUE_CUSTOMERS   => "HIGH_VALUE_CUSTOMERS",
  DISENGAGED_CUSTOMERS   => "DISENGAGED_CUSTOMERS",
  QUALIFIED_LEADS        => "QUALIFIED_LEADS",
  CONVERTED_LEADS        => "CONVERTED_LEADS",
  PAID_SUBSCRIBERS       => "PAID_SUBSCRIBERS",
  LOYALTY_SIGN_UPS       => "LOYALTY_SIGN_UPS",
  CART_ABANDONERS        => "CART_ABANDONERS",
  LOYALTY_TIER_1_MEMBERS => "LOYALTY_TIER_1_MEMBERS",
  LOYALTY_TIER_2_MEMBERS => "LOYALTY_TIER_2_MEMBERS",
  LOYALTY_TIER_3_MEMBERS => "LOYALTY_TIER_3_MEMBERS",
  LOYALTY_TIER_4_MEMBERS => "LOYALTY_TIER_4_MEMBERS",
  LOYALTY_TIER_5_MEMBERS => "LOYALTY_TIER_5_MEMBERS",
  LOYALTY_TIER_6_MEMBERS => "LOYALTY_TIER_6_MEMBERS",
  LOYALTY_TIER_7_MEMBERS => "LOYALTY_TIER_7_MEMBERS"
];

1;
