# Copyright 2020, Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

package Google::Ads::GoogleAds::V20::Enums::AdDestinationTypeEnum;

use strict;
use warnings;

use Const::Exporter enums => [
  UNSPECIFIED               => "UNSPECIFIED",
  UNKNOWN                   => "UNKNOWN",
  NOT_APPLICABLE            => "NOT_APPLICABLE",
  WEBSITE                   => "WEBSITE",
  APP_DEEP_LINK             => "APP_DEEP_LINK",
  APP_STORE                 => "APP_STORE",
  PHONE_CALL                => "PHONE_CALL",
  MAP_DIRECTIONS            => "MAP_DIRECTIONS",
  LOCATION_LISTING          => "LOCATION_LISTING",
  MESSAGE                   => "MESSAGE",
  LEAD_FORM                 => "LEAD_FORM",
  YOUTUBE                   => "YOUTUBE",
  UNMODELED_FOR_CONVERSIONS => "UNMODELED_FOR_CONVERSIONS"
];

1;
