# Copyright 2020, Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

package Google::Ads::GoogleAds::V19::Enums::ChangeClientTypeEnum;

use strict;
use warnings;

use Const::Exporter enums => [
  UNSPECIFIED                             => "UNSPECIFIED",
  UNKNOWN                                 => "UNKNOWN",
  GOOGLE_ADS_WEB_CLIENT                   => "GOOGLE_ADS_WEB_CLIENT",
  GOOGLE_ADS_AUTOMATED_RULE               => "GOOGLE_ADS_AUTOMATED_RULE",
  GOOGLE_ADS_SCRIPTS                      => "GOOGLE_ADS_SCRIPTS",
  GOOGLE_ADS_BULK_UPLOAD                  => "GOOGLE_ADS_BULK_UPLOAD",
  GOOGLE_ADS_API                          => "GOOGLE_ADS_API",
  GOOGLE_ADS_EDITOR                       => "GOOGLE_ADS_EDITOR",
  GOOGLE_ADS_MOBILE_APP                   => "GOOGLE_ADS_MOBILE_APP",
  GOOGLE_ADS_RECOMMENDATIONS              => "GOOGLE_ADS_RECOMMENDATIONS",
  SEARCH_ADS_360_SYNC                     => "SEARCH_ADS_360_SYNC",
  SEARCH_ADS_360_POST                     => "SEARCH_ADS_360_POST",
  INTERNAL_TOOL                           => "INTERNAL_TOOL",
  OTHER                                   => "OTHER",
  GOOGLE_ADS_RECOMMENDATIONS_SUBSCRIPTION =>
    "GOOGLE_ADS_RECOMMENDATIONS_SUBSCRIPTION"
];

1;
