# Copyright 2020, Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

package Google::Ads::GoogleAds::V19::Enums::AdGroupAdPrimaryStatusReasonEnum;

use strict;
use warnings;

use Const::Exporter enums => [
  UNSPECIFIED                       => "UNSPECIFIED",
  UNKNOWN                           => "UNKNOWN",
  CAMPAIGN_REMOVED                  => "CAMPAIGN_REMOVED",
  CAMPAIGN_PAUSED                   => "CAMPAIGN_PAUSED",
  CAMPAIGN_PENDING                  => "CAMPAIGN_PENDING",
  CAMPAIGN_ENDED                    => "CAMPAIGN_ENDED",
  AD_GROUP_PAUSED                   => "AD_GROUP_PAUSED",
  AD_GROUP_REMOVED                  => "AD_GROUP_REMOVED",
  AD_GROUP_AD_PAUSED                => "AD_GROUP_AD_PAUSED",
  AD_GROUP_AD_REMOVED               => "AD_GROUP_AD_REMOVED",
  AD_GROUP_AD_DISAPPROVED           => "AD_GROUP_AD_DISAPPROVED",
  AD_GROUP_AD_UNDER_REVIEW          => "AD_GROUP_AD_UNDER_REVIEW",
  AD_GROUP_AD_POOR_QUALITY          => "AD_GROUP_AD_POOR_QUALITY",
  AD_GROUP_AD_NO_ADS                => "AD_GROUP_AD_NO_ADS",
  AD_GROUP_AD_APPROVED_LABELED      => "AD_GROUP_AD_APPROVED_LABELED",
  AD_GROUP_AD_AREA_OF_INTEREST_ONLY => "AD_GROUP_AD_AREA_OF_INTEREST_ONLY",
  AD_GROUP_AD_UNDER_APPEAL          => "AD_GROUP_AD_UNDER_APPEAL"
];

1;
