# Copyright 2020, Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

package Google::Ads::GoogleAds::V18::Enums::RealEstatePlaceholderFieldEnum;

use strict;
use warnings;

use Const::Exporter enums => [
  UNSPECIFIED         => "UNSPECIFIED",
  UNKNOWN             => "UNKNOWN",
  LISTING_ID          => "LISTING_ID",
  LISTING_NAME        => "LISTING_NAME",
  CITY_NAME           => "CITY_NAME",
  DESCRIPTION         => "DESCRIPTION",
  ADDRESS             => "ADDRESS",
  PRICE               => "PRICE",
  FORMATTED_PRICE     => "FORMATTED_PRICE",
  IMAGE_URL           => "IMAGE_URL",
  PROPERTY_TYPE       => "PROPERTY_TYPE",
  LISTING_TYPE        => "LISTING_TYPE",
  CONTEXTUAL_KEYWORDS => "CONTEXTUAL_KEYWORDS",
  FINAL_URLS          => "FINAL_URLS",
  FINAL_MOBILE_URLS   => "FINAL_MOBILE_URLS",
  TRACKING_URL        => "TRACKING_URL",
  ANDROID_APP_LINK    => "ANDROID_APP_LINK",
  SIMILAR_LISTING_IDS => "SIMILAR_LISTING_IDS",
  IOS_APP_LINK        => "IOS_APP_LINK",
  IOS_APP_STORE_ID    => "IOS_APP_STORE_ID"
];

1;
